# Python imports
import time
from urllib import quote, unquote

# Zope imports
from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo, getSecurityManager

# CMF imports
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent


# CPS imports
from Products.CPSCore.CPSBase import CPSBaseDocument, CPSBase_adder    
from Products.CMFCore.CMFCorePermissions import ModifyPortalContent
from Products.CPSCore.EventServiceTool import getEventService

# ZWiki imports
from Products.ZWiki.ZWikiPage import ZWikiPage
from Products.ZWiki.CMFInit import initPageMetadata
from Products.ZWiki import Permissions
from Products.ZWiki.Defaults import LEAVE_PLACEHOLDER

# Class for an hack to solve id vs id() problem
class hackId(str):
    __allow_access_to_unprotected_subobjects__ = 1
    def __call__(self):
        return self

# Constructor and class of CPSZWiki product

def addCPSZWiki(container, id, title='', page_type=None, file='', REQUEST=None, **kw):
    """Add a CPSZWiki Document"""
    id = str(id)
    title = str(title)
    ob = CPSZWiki(id=id, source_string=file, __name__=id)
    ob.title = title
    ob.parents = []
    username = getSecurityManager().getUser().getUserName()
    ob.manage_addLocalRoles(username, ['Owner'])
    initPageMetadata(ob)
    ob.setPageType(
        page_type or getattr(container,'allowed_page_types',[None])[0])
    return CPSBase_adder(container, ob, REQUEST=REQUEST)
    
class CPSZWiki(CPSBaseDocument, ZWikiPage):
    """ZWiki adaptation to work with CPS"""
    meta_type = 'CPSZWiki'
    portal_type = meta_type
    security = ClassSecurityInfo()
    _properties = ZWikiPage._properties 
    manage_options = ZWikiPage.manage_options
    manage_main = ZWikiPage.manage_editForm
    view = ZWikiPage.view
    
    def __init__(self, id="", source_string='', mapping=None, __name__='', **kw):
        if id == "" and __name__ <> "":
            id = __name__
        if id <> "" and __name__ == "":
            __name__ = id
        ZWikiPage.__init__(self, source_string=source_string, mapping=mapping, __name__=__name__)
        CPSBaseDocument.__init__(self, id, **kw)
        # id/id() hack begin
        oldId = self.getId()
        del self.id
        self.id = hackId(oldId)
        self.__name__ = self.id()
        # id/id() hack end

    # id/id() hack begin    
    def __setattr__(self, key, value):
        if key == 'id':
            value = hackId(value)
            self._p_changed = 1
        self.__dict__.update({key : value})
    # id/id() hack end
    
    security.declareProtected(Permissions.View, 'pageWithId')
    def pageWithId(self,id,url_quoted=0,ignore_case=0):
        """
        Return the page in this folder (or in the catalog) with this id.

        It's the same code of ZWiki Product but each f[id] is replace by
        getattr(f, id) cause f[id] doesn't work  properly with CPS Proxy Folder

        """
        ALLBRAINS=0

        if id == None: return None # XXX review later
        if url_quoted: id = unquote(id)
        if self.linkToAllCataloged() or ALLBRAINS:
            page = self.pages(id=id) or (ignore_case and self.pages(id_nocase=id))
            return (page and page[0]) or None
        else:
            f = self.folder()
            # don't acquire
            # XXX special case: aq_base adds a REQUEST attribute
            # ('<Special Object Used to Force Acquisition>')
            if id in f.objectIds() and self.isZwikiPage(getattr(f, id)): # poor caching
                return getattr(f, id)
            elif ignore_case:
                id = string.lower(id)
                for i in self.pageIds():
                    if i.lower() == id:
                        return getattr(f, id)
            else:
                return None

    security.declarePublic('edit')
    def edit(self, REQUEST=None, **kw):
        """ edit """
        CPSBaseDocument.edit(self, **kw)
        if REQUEST is not None:
            page=REQUEST.get('page', None)
            text=REQUEST.get('text', None)
            type=REQUEST.get('type', None)
            title=REQUEST.get('title', '')
            timeStamp=REQUEST.get('timeStamp', None)
            subjectSuffix=REQUEST.get('subjectSuffix', '')
            log=REQUEST.get('log', '')
            check_conflict=REQUEST.get('check_conflict', 1) # temp (?)
            leaveplaceholder=REQUEST.get('leaveplaceholder', LEAVE_PLACEHOLDER)
            updatebacklinks=REQUEST.get('updatebacklinks', 1)
            subtopics=REQUEST.get('subptopics', None)
            ZWikiPage.edit(self, page=page, text=text, type=type, 
                           title=title, timeStamp=timeStamp, 
                           REQUEST=REQUEST, subjectSuffix=subjectSuffix, 
                           log=log, check_conflict=check_conflict, # temp (?)
                           leaveplaceholder=leaveplaceholder, 
                           updatebacklinks=updatebacklinks, 
                           subtopics=subtopics)
        self._p_changed = 1
        self.reindexObject()
        evtool = getEventService(self)
        # Note: usually we're not a proxy. The repository will listen
        # for this event and propagate it to the relevant proxies.
        evtool.notify('sys_modify_object', self, {})
    
    security.declareProtected(Permissions.View, '__call__')
    __call__ = ZWikiPage.__call__
    
    security.declareProtected(Permissions.View, 'SearchableText')
    SearchableText = ZWikiPage.SearchableText
    
InitializeClass(CPSZWiki)